#include <SDL.h>
#include <SDL_mixer.h>
#include <SDL_image.h>
#include <SDL_ttf.h>

#include <string>

#include "SDLgraph.h"

#include "tools.h"
#include "main.h"

using namespace std;


void TextOutSurface(int x, int y, string txt, Uint32 color, SDL_Surface* surf, TTF_Font* curfont)
{
	
	SDL_Color tmpfontcolor;
    Uint8 r,g,b;

    r=0;
    g=0;
    b=0;
    
	SDL_GetRGB(color,surf->format,&r,&g,&b);

    
    tmpfontcolor.r = r;
    tmpfontcolor.g = g;
    tmpfontcolor.b = b;
    tmpfontcolor.a = 255;

	SDL_Surface *resulting_text;
    
    resulting_text = TTF_RenderText_Blended(curfont, txt.data(), tmpfontcolor);

	SDL_Rect myRect;

	myRect.x=x;
	myRect.y=y;



	SDL_BlitSurface(resulting_text, NULL, surf, &myRect);

	SDL_FreeSurface(resulting_text);

}




SDL_Texture* LoadTexture(string filename)
{
	SDL_Surface *img = NULL;
	img = IMG_Load(filename.c_str());
	if (!img) error("IMG_Load "+filename);

	SDL_Texture *tex = NULL;
	tex = SDL_CreateTextureFromSurface(SDLren, img);
	SDL_FreeSurface(img);

	return tex;
}



SDL_Texture* LoadSprite(string filename)
{
	SDL_Surface *img = NULL;
	img = IMG_Load(filename.c_str());
	if (!img) error("IMG_Load "+filename);


	SDL_SetColorKey(img, 1, SDL_MapRGB(img->format,255,0,255));



    
    
	SDL_Texture *tex = NULL;
	tex = SDL_CreateTextureFromSurface(SDLren, img);
	SDL_FreeSurface(img);

	return tex;
}




Uint32 RGB32(Uint8 r, Uint8 g,Uint8 b)
{
	Uint32 color = (( r << 16 ) | ( g << 8 ) | b );

   if(SDL_BYTEORDER == SDL_BIG_ENDIAN) color =  SDL_Swap32(color);
   

	return color;
}

void GetRGB32(Uint32 c, Uint8 &r, Uint8 &g, Uint8 &b)
{
	Uint32 color=c;
    
	if(SDL_BYTEORDER == SDL_BIG_ENDIAN) color =  SDL_Swap32(color);

    color = SDL_Swap32(color);
    
	r = (( color >> 16 ) & 0xFF);
	g = ( (color >> 8) & 0xFF );
	b = (color & 0xFF );
}




void TextOut(int x, int y, string txt, Uint32 color, TTF_Font* curfont)
{
	SDL_Surface *surf = NULL;
	SDL_Color tmpfontcolor;
    Uint8 r,g,b;


    GetRGB32(color,r,g,b);
    


    
    tmpfontcolor.r = r;
    tmpfontcolor.g = g;
    tmpfontcolor.b = b;
    tmpfontcolor.a = 255;
    
	SDL_Surface *resulting_text;
    
    resulting_text = TTF_RenderText_Blended(curfont, txt.data(), tmpfontcolor);

	
    Uint32 rmask, gmask, bmask, amask;

    /* SDL interprets each pixel as a 32-bit number, so our masks must depend
       on the endianness (byte order) of the machine */
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;
#else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
    amask = 0xff000000;
#endif

    surf = SDL_CreateRGBSurface(SDL_SWSURFACE, resulting_text->w,resulting_text->h, 32,
                                   rmask, gmask, bmask, amask);
    if(surf == NULL) error("TextOut");


	SDL_GetRGB(color,surf->format,&r,&g,&b);

	SDL_FillRect(surf,NULL,SDL_MapRGBA(surf->format, 0, 0, 0, 255));

	SDL_BlitSurface(resulting_text, NULL, surf, NULL);

	SDL_FreeSurface(resulting_text);

	SDL_SetColorKey(surf, 1, SDL_MapRGB(surf->format,0,0,0));


	SDL_Texture *tex = NULL;
	tex = SDL_CreateTextureFromSurface(SDLren, surf);
	SDL_FreeSurface(surf);

	SDL_Rect myrect;

	SDL_QueryTexture(tex, NULL, NULL, &myrect.w, &myrect.h);
	myrect.x=x;
	myrect.y=y;

	SDL_RenderCopy(SDLren, tex, NULL, &myrect);
    
    SDL_DestroyTexture(tex);

}