#include "OSdefine.h"

#include <SDL.h>
#include <SDL_mixer.h>
#include <SDL_image.h>
#include <SDL_ttf.h>

#include <string>

#include "tools.h"
#include "SDLgraph.h"
#include "Node.h"
#include "SceneGraph.h"
#include "main.h"

#include "aShip.h"
#include "aShot.h"
#include "aStar.h"
#include "aAsteroid.h"


using namespace std;

int screenwidth=1280;
int screenheight=720;

SDL_Renderer *SDLren = NULL;
SceneGraph *myScene;
int curScore=0;
string _DATA_FOLDER_ = "data";

bool quit = false;
SDL_Event e;


int main(int argc, char** argv)
{

    
    _DATA_FOLDER_ = getPathFromFullFileName(pcharstring(argv[0]));
    
#ifdef OS_iOS
    
     _DATA_FOLDER_ = "";
    
#endif

#ifdef OS_iOS
    
     _DATA_FOLDER_ = ".";
    
#endif
    
    _DATA_FOLDER_+="/data";
    
    
	// if (SDL_Init(SDL_INIT_EVERYTHING) == -1) error("SDL_Init");
    
    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) == -1) error("SDL_Init");
    

	srand (SDL_GetTicks());

	SDL_Window *SDLwin = NULL;
    
    SDL_DisplayMode current;
    int tmpI = SDL_GetDesktopDisplayMode(0, &current);
    if (tmpI!=0) error("could not get Display info!");
    

#ifdef OS_WIN
	SDLwin = SDL_CreateWindow("StarCon MobileArcade", SDL_WINDOWPOS_UNDEFINED,
                             SDL_WINDOWPOS_UNDEFINED,
                             screenwidth,screenheight,
                             NULL);
#endif 
    
#ifdef OS_LINUX
	SDLwin = SDL_CreateWindow("StarCon MobileArcade", SDL_WINDOWPOS_UNDEFINED,
                             SDL_WINDOWPOS_UNDEFINED,
                             screenwidth,screenheight,
                             NULL);
#endif 

#ifdef OS_OSX
	SDLwin = SDL_CreateWindow("StarCon MobileArcade", SDL_WINDOWPOS_UNDEFINED,
                              SDL_WINDOWPOS_UNDEFINED,
                              screenwidth,screenheight,
                              NULL);
#endif
    
#ifdef OS_ANDROID
    
    screenwidth = current.w;
    screenheight = current.h;
    
	SDLwin = SDL_CreateWindow("StarCon MobileArcade", SDL_WINDOWPOS_UNDEFINED,
                             SDL_WINDOWPOS_UNDEFINED,
                             0,0,
                             SDL_WINDOW_FULLSCREEN_DESKTOP );
#endif
    
#ifdef OS_iOS
    
    screenwidth = current.w;
    screenheight = current.h;
    
	SDLwin = SDL_CreateWindow("StarCon MobileArcade", SDL_WINDOWPOS_UNDEFINED,
                              SDL_WINDOWPOS_UNDEFINED,
                              0,0,
                              SDL_WINDOW_FULLSCREEN_DESKTOP );
#endif
    
    
	if (!SDLwin) error("SDL_CreateWindow");	




	SDLren = SDL_CreateRenderer(SDLwin, -1, SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC); 
	if (!SDLren) error("SDL_CreateRenderer");

#ifdef OS_ANDROID
	SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "linear");  
	SDL_RenderSetLogicalSize(SDLren,screenwidth,screenheight);
#endif
#ifdef OS_iOS
	SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "linear");
	SDL_RenderSetLogicalSize(SDLren,screenwidth,screenheight);
#endif


	if (Mix_Init(0)==-1) error("MIX_Init fails");
	if (Mix_OpenAudio(44100,MIX_DEFAULT_FORMAT,2,1024)<0) error("Mix_OpenAudio fails");



	if (TTF_Init() < 0) error("TTF_Init fails: " + (string) SDL_GetError());  

	TTF_Font *myFont;
    string s=_DATA_FOLDER_+"/fonts/arial.ttf";
	if (!(myFont = TTF_OpenFont(s.c_str(), 36))) error("TTF_OpenFont36 'arial.ttf' fails: " + (string) SDL_GetError());
	
	TTF_Font *myFont2;
    s=_DATA_FOLDER_+"/fonts/arial.ttf";
	if (!(myFont2 = TTF_OpenFont(s.c_str(), 16))) error("TTF_OpenFont16 'arial.ttf' fails: " + (string) SDL_GetError());



	//////////////////////////////////

		SDL_Texture* img = LoadTexture(_DATA_FOLDER_+"/graph/starcon.bmp");

		
		SDL_SetRenderDrawColor(SDLren, 0, 0, 0, 255);
		SDL_RenderClear(SDLren);
		
		SDL_Rect myrect;
		SDL_QueryTexture(img, NULL, NULL, &myrect.w, &myrect.h);
		myrect.x=screenwidth / 2 - 320 / 2;
		myrect.y=screenheight / 2 - 20 / 2;
		SDL_RenderCopy(SDLren, img, NULL, &myrect);

		TextOut(screenwidth / 2 + 72, screenheight / 2 + 10 , "mobilArcade",0xffffffff, myFont2);
		
		SDL_RenderPresent(SDLren);

		while (!quit)
		{
			while (SDL_PollEvent(&e))
			{
				switch(e.type) 
				{
					case SDL_QUIT: { quit = true; } break;
					case SDL_APP_TERMINATING: { quit = true; } break;
					case SDL_WINDOWEVENT_CLOSE: { quit=true; } break;
					case SDL_KEYDOWN: { quit=true; } break;
					case SDL_FINGERDOWN: { quit=true; } break;
					case SDL_MOUSEBUTTONDOWN: { quit=true; } break;
					default: break;				
				}
			}
		}

    SDL_DestroyTexture(img);
	//////////////////////////////////


    myScene = new SceneGraph();


	LoadStarImageData();
	for (int i=0;i<maxstars;i++)
	{
		aStar* myStar = new aStar();
		myScene->addNode(myStar);
	}


	LoadAsteroidImageData();
	aAsteroid* myAsteroid = new aAsteroid();
	myScene->addNode(myAsteroid);


	aShip *myShip = new aShip();
	myScene->addNode(myShip);


	struct curData
	{
		Uint32 ShipX;
		Uint32 shoot;
	};

	curData myCurData;

	int x=100;
	//int y=100;
	
	bool right=false;
	bool left=false;
	bool fire=false;
	bool space=false;



	
	quit=false;


	while (!quit)
	{
		//Event polling
		while (SDL_PollEvent(&e))
		{

			switch(e.type) 
			{
				case SDL_QUIT: { quit = true; } break;
				case SDL_APP_TERMINATING: { quit = true; } break;
				case SDL_WINDOWEVENT_CLOSE: { quit=true; } break;
				
#ifdef OS_TOUCH
				case SDL_FINGERMOTION:
				{
					x= e.tfinger.x * screenwidth;
				} break;
				case SDL_FINGERDOWN:
				{
					space=true;
                    
                    int curX=e.tfinger.x * screenwidth;
                    int curY=e.tfinger.y * screenheight;
                    if ((curX>screenwidth-100)&&(curX<screenwidth)&&(curY>0)&&(curY<40)) quit=true; break;
                    
				} break;
				case SDL_FINGERUP:
				{
					space=false;
					fire=false;
				} break;
#endif
#ifdef OS_NO_TOUCH
                case SDL_KEYDOWN:
                {
                    switch (e.key.keysym.sym)
                    {
                        case SDLK_ESCAPE: {quit = true;} break;
                        case SDLK_RIGHT: {right = true;} break;
                        case SDLK_LEFT: {left = true;} break;
                        case SDLK_SPACE: { space=true; } break;
                        default: break;
                    }
                } break;
                    
				case SDL_KEYUP:
				{
                    switch (e.key.keysym.sym)
					{
						case SDLK_RIGHT: {right = false;} break;
						case SDLK_LEFT: {left = false;} break;
						case SDLK_SPACE: {fire=false; space=false;} break;
						default: break;
					}
				} break;
				case SDL_MOUSEMOTION:
				{
					x=e.motion.x;
				} break;
                    
				case SDL_MOUSEBUTTONDOWN:
				{
					if (e.button.state == SDL_BUTTON_LMASK) space=true;
				} break;
                    
				case SDL_MOUSEBUTTONUP:
				{
					space=false;
					fire=false;
				} break;
#endif

				default: break;
			}

			
		}


		if (right)x++;
		if (left)x--;

		if (x>screenwidth-80) x=screenwidth-80;

		if (!fire && space) 
		{
			myCurData.shoot=1;
			fire=true;
		} else myCurData.shoot = 0;


		myCurData.ShipX = x;



		myScene->update((void*)&myCurData);
		


		SDL_SetRenderDrawColor(SDLren, 0, 0, 0, 255);
		SDL_RenderClear(SDLren);

		myScene->render();


		 TextOut(10, 10, numstr(curScore),0xffffffff, myFont);
        
        #ifdef OS_TOUCH
         TextOut(screenwidth - 100, 10, "quit",0xffffffff, myFont);
        #endif
        
		SDL_RenderPresent(SDLren);


	}


	 FreeStarData();
	 FreeAsteroidData();


	SDL_DestroyRenderer(SDLren);
	SDL_DestroyWindow(SDLwin);
	

	
	TTF_CloseFont(myFont);
	TTF_CloseFont(myFont2);
	
	
	TTF_Quit();
	Mix_CloseAudio();
	SDL_Quit();
	
    myScene->closeNodes();
    delete myScene;
    
    
	logwrite("normal Exit");



    exit(0);

	return 0;
}
